import numpy as np
import cv2 as cv
import matplotlib.pyplot as plt


def chunks(lst, n):
    """Yield successive n-sized chunks from lst."""
    for i in range(0, len(lst), n):
        yield lst[i:i + n]


# params for ShiTomasi corner detection
feature_params = dict(maxCorners=1000,
                      qualityLevel=0.0001,
                      minDistance=4,
                      blockSize=50)

# Parameters for lucas kanade optical flow
lk_params = dict(winSize=(100, 100),
                 maxLevel=200,
                 criteria=(cv.TERM_CRITERIA_EPS | cv.TERM_CRITERIA_COUNT, 10, 0.03))

# Create some random colors for optical flow
color = np.random.randint(0, 255, (10000, 3))


# Loop over all 7 videos in "resources/on" directory
# Videos are named 1.mp4, 2.mp4, ...
# Each Video containts a single gait cycle
for index in range(1, 8):
    file = cv.samples.findFile("resources/on/" + str(index) + ".mp4")

    backSub = cv.createBackgroundSubtractorKNN()

    # Create a background substraction mask from the video
    cap = cv.VideoCapture(file)
    fgMask = []
    while(1):
        ret, frame = cap.read()
        if frame is None:
            break
        frame_gray = cv.cvtColor(frame, cv.COLOR_BGR2GRAY)
        fgMask = backSub.apply(frame_gray)

    # Now that we have the background mask, do rest of the processing
    cap = cv.VideoCapture(file)

    # Take the first frame
    ret, old_frame = cap.read()
    old_gray = cv.cvtColor(old_frame, cv.COLOR_BGR2GRAY)

    # Apply background mask and then find good points to track from the result
    masked_frame = cv.bitwise_or(old_gray, old_gray, mask=fgMask)
    p0 = cv.goodFeaturesToTrack(masked_frame, mask=None, **feature_params)

    # Create a mask image to draw optical flow points
    mask = np.zeros_like(old_frame)

    # Variable to save polar coordinates
    polars = []

    while(1):
        ret, frame = cap.read()

        # Stop the while loop if no more frames left
        if frame is None:
            break

        # Create gray scale frame for optical flow algorithm
        frame_gray = cv.cvtColor(frame, cv.COLOR_BGR2GRAY)

        # calculate optical flow
        p1, st, err = cv.calcOpticalFlowPyrLK(
            old_gray, frame_gray, p0, None, **lk_params)

        # Select good points
        good_new = p1[st == 1]
        good_old = p0[st == 1]

        # Draw the optical flow points and make polar coordinates out of the delta
        for i, (new, old) in enumerate(zip(good_new, good_old)):
            a, b = new.ravel()
            c, d = old.ravel()
            mask = cv.line(mask, (a, b), (c, d), color[i].tolist(), 2)
            # frame = cv.circle(frame, (a, b), 5, color[i].tolist(), -1)

            # Polar conversion
            u = a-c
            v = b-d
            r = np.sqrt(np.power(u, 2) + np.power(v, 2))
            theta = 0
            if u == 0 and v == 0:
                theta = 0
            elif u >= 0:
                theta = np.arcsin(v/r)
            else:
                theta = np.arcsin(v/r) * (-1) + np.pi

            # Store the polar coordinates
            polars.append((r, theta))

        # Display the result
        optical_flow_img = cv.add(frame, mask)
        cv.imshow('foreground mask', optical_flow_img)
        rgb_mask = cv.cvtColor(fgMask, cv.COLOR_GRAY2BGR)
        combined_img = np.concatenate(
            (frame, optical_flow_img, rgb_mask), axis=1)
        cv.imshow('frame', combined_img)

        k = cv.waitKey(30) & 0xff
        if k == 27:
            break

        # Now update the previous frame and previous points
        old_gray = frame_gray.copy()
        p0 = good_new.reshape(-1, 1, 2)

    # Average and write to file the recorded polar coordinates
    with open('polar_coords.data', 'a') as gait_data_file:
        # chunks
        chunk_len = 400
        for chunk in chunks(polars, chunk_len):
            if len(chunk) == chunk_len:
                x = np.average(chunk, 0)[0]
                y = np.average(chunk, 0)[1]
                gait_data_file.write(str(x) + "," + str(y) + ",abnormal\n")
